import stringify from "json-stable-stringify";
import objectHash from "object-hash";

export const buildMatches = (
  matches: {
    id: number;
    messageId: number;
    guildId: number;
    channelId: number;
    createrId: number;
    roleId: number;
    matchType: string;
    opponentName: string;
    utc_ts: Date;
  }[],
) =>
  matches.map(
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    ({ id, guildId, utc_ts, ...match }) => ({
      ...match,
      utc_ts: utc_ts.toISOString(),
    }),
  );

export function buildConfig(guildQuery: {
  id: number;
  timezone: string;
  tpEnabled: boolean;
  tpChannelId: number | null;
  tpInterval: number;
  tpRoles: boolean;
  isAvailableRoleId: number | null;
  wantsToBeNotifieRoledId: number | null;
  tpMessages: {
    messageId: number | null;
    day: number;
    guildId: number;
  }[];
  matches: {
    id: number;
    messageId: number;
    guildId: number;
    channelId: number;
    createrId: number;
    roleId: number;
    matchType: string;
    opponentName: string;
    utc_ts: Date;
  }[];
}) {
  const {
    id,
    timezone,
    tpEnabled,
    tpChannelId,
    tpInterval,
    tpRoles,
    isAvailableRoleId,
    wantsToBeNotifieRoledId,
  } = guildQuery;

  const targetMinute = tpInterval & 63;
  const targetHour = (tpInterval >> 6) & 31;
  const targetDay = (tpInterval >> 11) & 7;

  const payload = {
    guildId: id,
    timezone,
    features: {
      timePlanning: {
        enabled: tpEnabled,
        channelId: tpChannelId,
        targetMinute,
        targetHour,
        targetDay,
        roles: { enabled: tpRoles, isAvailableRoleId, wantsToBeNotifieRoledId },
      },
    },
    matches: buildMatches(guildQuery.matches),
    checksum: objectHash(stringify(guildQuery)),
  };
  return payload;
}
